@@ -29,11 +29,15 @@ import gtk
 import gobject
 import hildon
 import sys
+import pygame
 from time import strftime
-from datetime import datetime
+from datetime import datetime, timedelta
 
 from portrait import FremantleRotation
 
+STOPWATCH = 1
+COUNTDOWN = 2
+
 class StopwatchApp(hildon.Program):
     def __init__(self):
         hildon.Program.__init__(self)
@@ -46,6 +50,10 @@ class StopwatchApp(hildon.Program):
         self.startButton = None
         self.checkpointButton = None
         self.clearButton = None   
+        self.hourButton = None
+        self.minuteButton = None
+        self.secondButton = None
+        self.mode = STOPWATCH
         self.cleared = True    
         self.savedTimings = []
       
@@ -59,6 +67,22 @@ class StopwatchApp(hildon.Program):
 
         self.add_window(self.window)
        
+        # create buttons
+        self.create_buttons()
+        
+        # create label        
+        self.timeLabel = gtk.Label()
+        self.clear_time()
+        
+        # load beep
+        pygame.mixer.init()
+        self.sound = pygame.mixer.Sound("/usr/share/sounds/ui-default_beep.wav")
+        
+        self.drawUI()
+        menu = self.create_menu(None)
+        self.window.set_app_menu(menu)
+   
+    def create_buttons(self):
         # buttons        
         # stop button
         self.stopButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
@@ -96,23 +120,42 @@ class StopwatchApp(hildon.Program):
         self.clearButton.set_image_position(gtk.POS_LEFT)
         self.clearButton.set_sensitive(False)
         
+        r = []
+        for i in range(59):
+            r.append("%s" % i) 
+        # hour button
+        self.hourButton = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,hildon.BUTTON_ARRANGEMENT_VERTICAL)      
+        self.hourButton.set_label("Hours")
+        self.hourButton.set_title("Hours")
+        self.hourButton.set_selector(self.create_customized_selector(r))    
+        self.hourButton.set_value("0")   
+
+        # minute button
+        self.minuteButton = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,hildon.BUTTON_ARRANGEMENT_VERTICAL)      
+        self.minuteButton.set_label("Minutes")
+        self.minuteButton.set_title("Minutes")
+        self.minuteButton.set_selector(self.create_customized_selector(r))    
+        self.minuteButton.set_value("0")   
         
-        # create clickable label        
-        self.timeLabel = gtk.Label()
-        self.clear_time()
-        
-        self.eventBox = gtk.EventBox()
-        self.eventBox.add(self.timeLabel)
-        self.eventBox.connect("button-press-event", self.label_clicked, None)
-        
-        self.drawUI()
-        menu = self.create_menu(None)
-        self.window.set_app_menu(menu)
-        
-    def drawUI(self, screen=None):      
+        # second button
+        self.secondButton = hildon.PickerButton(gtk.HILDON_SIZE_AUTO,hildon.BUTTON_ARRANGEMENT_VERTICAL)      
+        self.secondButton.set_label("Seconds")
+        self.secondButton.set_title("Seconds")
+        self.secondButton.set_selector(self.create_customized_selector(r))    
+        self.secondButton.set_value("0")   
+                
+    def create_customized_selector(self, data):
+        # Create a touch selector 
+        selector = hildon.TouchSelector(text=True)
+                       
+        for item in data:
+            selector.append_text(item)
+        return selector
+     
+    def drawUI(self, a=None):      
         # clear everything
         def clearGtkTree(item):
-            if (type(item) is gtk.EventBox) or (type(item) is hildon.GtkButton):
+            if (type(item) is gtk.EventBox) or (type(item) is hildon.GtkButton) or (type(item) is hildon.PickerButton):
                 return
             try:
                 for child in item.get_children():            
@@ -125,41 +168,68 @@ class StopwatchApp(hildon.Program):
             
         if (self.rotation._orientation) == "portrait":
             # buttons vbox
-            bvbox = gtk.VBox(True, 0)
-            bvbox.add(self.startButton)
-            bvbox.add(self.stopButton)
-            bvbox.add(self.checkpointButton)
-            bvbox.add(self.clearButton)
+            if self.mode == STOPWATCH:
+                bvbox = gtk.VBox(True, 0)
+                bvbox.add(self.startButton)
+                bvbox.add(self.stopButton)
+                bvbox.add(self.checkpointButton)
+                bvbox.add(self.clearButton)
+            else:
+                bvbox = gtk.VBox(True, 0)
+                bvbox.add(self.startButton)
+                bvbox.add(self.stopButton)
+                bvbox.add(self.hourButton)
+                bvbox.add(self.minuteButton)
+                bvbox.add(self.secondButton)
             
             vbox = gtk.VBox(False, 10)
-            vbox.add(self.eventBox)
+            vbox.add(self.timeLabel)
             vbox.add(bvbox)
             
             self.window.add(vbox)
             
         else:
-            # buttons hbox
-            bhbox = gtk.HBox(True, 0)
+            if self.mode == STOPWATCH:
+                # buttons hbox
+                bhbox = gtk.HBox(True, 0)
+                
+                bvbox = gtk.VBox(True, 0)
+                bvbox.add(self.startButton)
+                bvbox.add(self.checkpointButton)
+                bhbox.add(bvbox)
+                bvbox = gtk.VBox(True, 0)
+                bvbox.add(self.stopButton)
+                bvbox.add(self.clearButton)
+                bhbox.add(bvbox)
+                vbox = gtk.VBox(False, 10)
+                vbox.add(self.timeLabel)
+                vbox.add(bhbox)
+                
+                self.window.add(vbox)
+                
+            elif self.mode == COUNTDOWN:
+                vhbox = gtk.VBox(True, 0)
+                bhbox = gtk.HBox(True, 0)
+                bhbox.add(self.startButton)
+                bhbox.add(self.stopButton)
+                
+                vhbox.add(bhbox)
+                bhbox = gtk.HBox(True, 0)             
+                bhbox.add(self.hourButton)
+                bhbox.add(self.minuteButton)
+                bhbox.add(self.secondButton)
+                vhbox.add(bhbox)
+
+                vbox = gtk.VBox(False, 10)
+                vbox.add(self.timeLabel)
+                vbox.add(vhbox)
             
-            bvbox = gtk.VBox(True, 0)
-            bvbox.add(self.startButton)
-            bvbox.add(self.checkpointButton)
-            bhbox.add(bvbox)
-            bvbox = gtk.VBox(True, 0)
-            bvbox.add(self.stopButton)
-            bvbox.add(self.clearButton)
-            bhbox.add(bvbox)
-            
-            vbox = gtk.VBox(False, 10)
-            vbox.add(self.eventBox)
-            vbox.add(bhbox)
-            
-            self.window.add(vbox)
+                self.window.add(vbox)
+
           
         self.update_time()  
         self.window.show_all()
          
-
     def update_time(self):                      
         if self.running:
             now = datetime.now()
@@ -167,7 +237,11 @@ class StopwatchApp(hildon.Program):
             now = self.stopTime          
         
         try:
-            timeString = str(now - self.startTime)
+            if self.mode == STOPWATCH:
+                timeString = str(now - self.startTime)
+            else:
+                timeString = str(self.stopTime - now)
+                
         except TypeError:
             timeString = "0:00:00.000000"
             
@@ -176,7 +250,30 @@ class StopwatchApp(hildon.Program):
         
         if self.running:
             gobject.timeout_add(50, self.update_time)
-
+            
+    def checkCountdown(self):
+        now = datetime.now()
+        
+        if self.stopTime == None:
+            return
+        
+        if now >= self.stopTime:
+#            self.player.set_state(gst.STATE_PLAYING)
+            self.sound.play()
+            self.running = False
+            hildon.hildon_banner_show_information(self.window, "", "Timer finished at %s" % str(now)[:-7])
+            self.stopButton.set_sensitive(False)
+            self.startButton.set_sensitive(True)
+            self.hourButton.set_sensitive(True)
+            self.minuteButton.set_sensitive(True)
+            self.secondButton.set_sensitive(True)
+            self.startTime = None
+            self.stopTime = None
+            
+            
+        else:
+            gobject.timeout_add(10, self.checkCountdown)
+ 
     def clear_time(self):
         fontSize = self.get_font_size()
         timeString = "0:00:00.000000"
@@ -212,7 +309,6 @@ class StopwatchApp(hildon.Program):
         self.checkpointButton.set_sensitive(True)
 
     def quit(self, *args):
-        print "Goodbye world!"
         gtk.main_quit()
         
     def run(self):     
@@ -222,81 +318,117 @@ class StopwatchApp(hildon.Program):
     def menu_button_clicked(self, button, lbale):        
         label = button.get_label()
         if label == "Go":
-            if self.running == False:
-                self.running = True
-                if self.cleared == True:
-                    self.startTime = datetime.now()   
-                else:
-                    self.startTime = datetime.now() - (self.stopTime - self.startTime)                                  
+            if self.mode == STOPWATCH:
+                if self.running == False:
+                    self.running = True
+                    if self.cleared == True:
+                        self.startTime = datetime.now()   
+                    else:
+                        self.startTime = datetime.now() - (self.stopTime - self.startTime)                                  
+                    
+                    self.update_time()                
+                    self.stopButton.set_sensitive(True)
+                    self.set_checkpoint_button()
+                    self.clearButton.set_sensitive(False)
+                    self.startButton.set_sensitive(False)
+                    self.cleared = False
+                    self.running = True
+                elif self.running == True:
+                    self.clearButton.set_sensitive(False)
+            
+            elif self.mode == COUNTDOWN:
+                
+                hour = int(self.hourButton.get_value())
+                minute = int(self.minuteButton.get_value())
+                second = int(self.secondButton.get_value())
+                
+                countdownDelta = timedelta(hours = hour, minutes = minute, seconds = second)
                 
-                self.update_time()                
                 self.stopButton.set_sensitive(True)
-                self.set_checkpoint_button()
-                self.clearButton.set_sensitive(False)
+                self.hourButton.set_sensitive(False)
+                self.minuteButton.set_sensitive(False)
+                self.secondButton.set_sensitive(False)
                 self.startButton.set_sensitive(False)
-                self.cleared = False
                 self.running = True
-            elif self.running == True:
-                self.clearButton.set_sensitive(False)
+                self.startTime = datetime.now()
+                self.stopTime = self.startTime + countdownDelta
+                self.update_time()
+                self.checkCountdown()
+                
         
         elif label == "Stop":
-                self.running = False  
+            self.running = False  
+            
+            if self.mode == STOPWATCH:
                 self.stopTime = datetime.now()
                 self.update_time()
                 self.clearButton.set_sensitive(True)
                 self.startButton.set_sensitive(True)
                 self.stopButton.set_sensitive(False)
                 self.set_checkpointlist_button()
-
+            elif self.mode == COUNTDOWN:
+                self.hourButton.set_sensitive(True)
+                self.minuteButton.set_sensitive(True)
+                self.secondButton.set_sensitive(True)
+                self.stopButton.set_sensitive(False)
+                self.startButton.set_sensitive(True)
+                self.stopTime = None
+                
                 
         elif label == "Clear":
             self.clear_time()
             
         elif label == "Checkpoint":
-            self.checkpoint()
+            self.add_checkpoint()
             
         elif label == "Checkpoint List":
-            self.label_clicked()
-        
+            self.list_checkpoints()      
+            
+        elif label == "Stopwatch":
+            self.window.set_title("Stopwatch")
+            self.mode = STOPWATCH
+            self.clear_time()
+            self.drawUI()
             
         elif label == "Countdown":
-            self.stopButton = hildon.TimeButton(gtk.HILDON_SIZE_AUTO, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-            self.stopButton.set_text("Pick time", "")
-   
-
-    def checkpoint(self):
+            self.window.set_title("Countdown")
+            self.mode = COUNTDOWN
+            self.clear_time()
+            self.drawUI()
+            
+        elif label == "Set":
+            pass
+        
+    def add_checkpoint(self):
         now = datetime.now()
         self.savedTimings.append(now-self.startTime)            
         hildon.hildon_banner_show_information(self.window, "", "Saved time: %s" % str(now-self.startTime)[:-4])
         
-    def label_clicked(self, a=None, b=None, c=None):     
-        if self.running:    
-            self.checkpoint()
-        else:
-            list = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
-            
-            counter = 1
-            for i in self.savedTimings:
-                niter = list.append()
-                list.set(niter, 0, "# %d -" % counter, 1, str(i)[:-4])
-                counter += 1
-            
-            win = hildon.StackableWindow()
-            win.set_title("Saved Times")  
-            vbox = gtk.VBox()
-            parea = hildon.PannableArea()
-            
-            #Define the recipe list treeview with multiple selections, search, etc.
-            self.checkpoint_tv = hildon.GtkTreeView(gtk.HILDON_UI_MODE_EDIT)
-            selection = self.checkpoint_tv.get_selection()
-            self.checkpoint_tv.set_model(list)
-    
-            parea.add(self.checkpoint_tv)
-            # add columns to the tree view
-            self.add_columns_to_checkpoint_list(self.checkpoint_tv)
-            vbox.pack_start(parea, True, True, 0)
-            win.add(vbox)          
-            win.show_all()
+    def list_checkpoints(self):     
+        list = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
+        
+        counter = 1
+        for i in self.savedTimings:
+            niter = list.append()
+            list.set(niter, 0, "# %d -" % counter, 1, str(i)[:-4])
+            counter += 1
+        
+        win = hildon.StackableWindow()
+        win.set_title("Saved Times")  
+        vbox = gtk.VBox()
+        parea = hildon.PannableArea()
+        
+        #Define the recipe list treeview with multiple selections, search, etc.
+        self.checkpoint_tv = hildon.GtkTreeView(gtk.HILDON_UI_MODE_EDIT)
+        selection = self.checkpoint_tv.get_selection()
+        self.checkpoint_tv.set_model(list)
+
+        parea.add(self.checkpoint_tv)
+        # add columns to the tree view
+        self.add_columns_to_checkpoint_list(self.checkpoint_tv)
+        vbox.pack_start(parea, True, True, 0)
+        win.add(vbox)          
+        win.show_all()
 
     def add_columns_to_checkpoint_list(self, treeview):
         model = treeview.get_model()
@@ -325,13 +457,11 @@ class StopwatchApp(hildon.Program):
         menu.append(button)
         button.set_mode(False)
         
-#===============================================================================
-#        button = hildon.GtkRadioButton(gtk.HILDON_SIZE_AUTO, button)
-#        button.set_label("Countdown")
-#        button.connect("clicked", self.menu_button_clicked, label)
-#        menu.append(button)
-#        button.set_mode(False)
-#===============================================================================
+        button = hildon.GtkRadioButton(gtk.HILDON_SIZE_AUTO, button)
+        button.set_label("Countdown")
+        button.connect("clicked", self.menu_button_clicked, label)
+        menu.append(button)
+        button.set_mode(False)
         
         # about button
         button = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
